--------------------------------------------------------------------------------
-- Carnage Contest Game Mode Script
-- 3 Rounds Domination, Holzchopf, 2013
-- v1.0 - 24 Sep 2013	- Created
--------------------------------------------------------------------------------
addgamemode("dom3","3 Rounds Domination")

dom3_gmvar_hasground=true
dom3_gmvar_score={0,0,0,0,0,0,0,0}
dom3_gmvar_health={}
dom3_gmvar_spawn={}
dom3_gmvar_gfxborder=loadgfx("holzchopf/domination_border.png")
setmidhandle(dom3_gmvar_gfxborder)

function dom3_gm_start()
	dom3_gmvar_hasground=true
	players=playertable(0,0)
	-- remember player initial health
	for i=1,#players do
		dom3_gmvar_health[i]=getplayerhealth(players[i])
	end
	-- reset score
	dom3_gmvar_score={0,0,0,0,0,0,0,0}
	-- set up checkpoints
	holzchopf.checkpoint.projrad=30
	holzchopf.checkpoint.plrrad=30
	-- create checkpoints
	local objs=objecttable(holzchopf.checkpoint.id)
	if #objs==0 then
		local xpos={0.1,0.9,0.1,0.9}
		local ypos={0.1,0.1,0.7,0.7}
		for i=1,4 do
			local set=0
			x=xpos[i]*getmapwidth()
			y=ypos[i]*getwatery()
			createobject(holzchopf.checkpoint.id,x,y)
			-- this is nescessary, because the recently created objects
			-- do not occur in objecttable -_-
			if collision(col30x30,x,y,1,0,0)==1 then
				terrainrect(x-30,y-30,60,60,0x00000000)
				terrainalphaimage(dom3_gmvar_gfxborder,x,y)
			end
		end
	end
	-- delete terrain near to checkpoints
	objs=objecttable(holzchopf.checkpoint.id)
	for i=1,#objs do
		x=getobjectx(objs[i])
		y=getobjecty(objs[i])
		if collision(col30x30,x,y,1,0,0)==1 then
			terrainrect(x-30,y-30,60,60,0x00000000)
			terrainalphaimage(dom3_gmvar_gfxborder,x,y)
		end
		-- this is nescessary for items created in map editor
		objects[objs[i]].alliance=holzchopf.checkpoint.alliance
		objects[objs[i]].projrad=holzchopf.checkpoint.projrad
		objects[objs[i]].plrrad=holzchopf.checkpoint.plrrad
	end
end

function dom3_gm_draw()
	-- sorted score
	score={}
	for i=1,8 do
		score[i]={}
		score[i].scr=dom3_gmvar_score[i]
		score[i].all=i
	end
	table.sort(score,function(a,b) return a.scr>b.scr end)
	-- draw score
	setscale(0.5,0.5)
	setrotation(0)
	for y=1,8 do
		if score[y].scr>0 then
			if		score[y].all==0 then setcolor(255,255,255)
			elseif	score[y].all==1 then setcolor(000,255,000)
			elseif	score[y].all==2 then setcolor(255,255,000)
			elseif	score[y].all==3 then setcolor(255,000,000)
			elseif	score[y].all==4 then setcolor(080,200,255)
			elseif	score[y].all==5 then setcolor(180,060,255)
			elseif	score[y].all==6 then setcolor(255,180,000)
			elseif	score[y].all==7 then setcolor(000,180,000)
			elseif	score[y].all==8 then setcolor(050,100,255)
			end
			for x=1,score[y].scr do
				drawhud(holzchopf.checkpoint.gfx_in,x*16-8,12+y*16)
				drawhud(holzchopf.checkpoint.gfx_out,x*16-8,12+y*16)
			end
		end
	end
end

function dom3_gm_update()
	-- check for drowning players
	players=playertable(0,0)
	local p,y
	for i=1,#players do
		p=players[i]
		y=getplayery(p)+getplayeryspeed(p)+32
		-- player drowns
		if y>=getwatery() then
			-- fx
			x=getplayerx(p)
			y=getplayery(p)
			particle(p_waterhit,x,y)
			-- curent player drowns
			if p==playercurrent() and getframesleft()>0 then
				endturn()
				playercontrol(0)
			end
			-- spawn
			dom3_gm_spawn(p)
		end
		-- player died otherwise
		if dom3_gmvar_spawn[p]==1 then
			-- fx
			x=getplayerx(p)
			y=getplayery(p)
			particle(p_bodypart,x,y)
			for j=1,3 do
				particle(p_meat,x,y)
			end
			particle(p_bigblood,x,y)
			-- current player died
			if p==playercurrent() and getframesleft()>0 then
				endturn()
				playercontrol(0)
			end
			-- spawn
			dom3_gm_spawn(p)
			dom3_gmvar_spawn[p]=0
		end
	end
end

function dom3_gm_playerdamage(player,damage)
	-- check for kill
	if getplayerhealth(player)-damage<=1 then
		-- remember to spawn (can't be done directly, because terrainexplosion
		-- sets playerposition)
		dom3_gmvar_spawn[player]=1
		return -1
	-- damage won't kill player
	else
		return 0
	end
end

function dom3_gm_endturn()
	-- distribute score
	local objs=objecttable(holzchopf.checkpoint.id)
	for i=1,#objs do
		local a=objects[objs[i]].alliance
		if a>0 then
			dom3_gmvar_score[a]=dom3_gmvar_score[a]+1
		end
	end
	-- end of round 3
	if getround()>=3 and getturn(1)==getturn(2) then
		local winalliance=-1
		local winscore=-1
		-- find alliance with best score
		for i=1,8 do
			if dom3_gmvar_score[i]>winscore then
				winscore=dom3_gmvar_score[i]
				winalliance=i
			elseif dom3_gmvar_score[i]==winscore then
				winalliance=-1
			end
		end
		return winalliance
	else
		-- Let the game decide itself if it's over
		return -2
	end
end

function dom3_gm_spawn(player)
	-- check if map has still some ground
	if dom3_gmvar_hasground==false then
		return
	end
	-- place player randomly, try 100 times, abort afterwards
	local placed=false
	local i=1
	randomseed(getturn())
	repeat
		x=random(10,getmapwidth()-10)
		y=random(20,getwatery()-32)
		-- find plain terrain spot
		if collision(colplayer,x,y,1,0,0)==1 then
			-- go up, so the player isn't placed sub-terrain
			repeat
				y=y-1
				collision(colplayer,x,y)
				if playercollision()~=0 or objectcollision()~=0 then
					break
				elseif terraincollision()==0 then
					playerposition(player,x,y)
					placed=true
					break
				end
			until placed==true
		-- abort after 100 tries
		elseif i==100 then
			dom3_gmvar_hasground=false
			return
		end
		i=i+1
	until placed==true
	-- neutralize player
	playerpush(player,0,0,1)
	playerstate(player,state_poisoned,0)
	playerstate(player,state_confused,0)
	playerstate(player,state_frozen,0)
	playerstate(player,state_sleeping,1)
	playerstate(player,state_invisible,0)
	-- set player health to default
	local h=getplayerhealth(player)-dom3_gmvar_health[player]
	if h>0 then
		playerdamage(player,h)
	elseif h<0 then
		playerheal(player,-h)
	end
	-- fx
	if player==playercurrent() then
		scroll(x,y)
	end
	particle(p_muzzle,x,y)
	particlesize(1,1)
	particlecolor(0,144,255)
	particlealpha(1.0)
	particlefadealpha(0.01)
	particle(p_muzzle,x,y)
	particlesize(1,3)
	particlecolor(0,204,255)
	particlealpha(1.0)
	particlefadealpha(0.02)
	particlerotation(0)
	for j=1,20 do
		particle(p_flare,x-10.0+math.random()*20.0,y+math.random()*10.0)
		particlesize(3.0,3.0)
		particlecolor(255,255,255)
		particlespeed(0,math.random(10,30)*-0.05)
		particlefadealpha(0.01)
	end
end

--------------------------------------------------------------------------------
-- Carnage Contest Checkpoint Object, Holzchopf, 2013
-- v1.0 - 24 Sep 2013	- Created
--------------------------------------------------------------------------------

-- setup
if holzchopf==nil then holzchopf={} end
holzchopf.checkpoint={}
holzchopf.checkpoint.alliance=0
holzchopf.checkpoint.projrad=0
holzchopf.checkpoint.plrrad=0

-- Load & Prepare Ressources
holzchopf.checkpoint.gfx_in=loadgfx("holzchopf/checkpoint_in.png")
setmidhandle(holzchopf.checkpoint.gfx_in)
holzchopf.checkpoint.gfx_out=loadgfx("holzchopf/checkpoint_out.png")
setmidhandle(holzchopf.checkpoint.gfx_out)
holzchopf.checkpoint.sfx_check=loadsfx("teleport.wav")

--------------------------------------------------------------------------------
-- Object
--------------------------------------------------------------------------------
holzchopf.checkpoint.id=addobject("holzchopf.checkpoint",0)

function holzchopf.checkpoint.setup(id,parameter)
	if objects==nil then objects={} end
	objects[id]={}
	objects[id].alliance=holzchopf.checkpoint.alliance
	objects[id].projrad=holzchopf.checkpoint.projrad
	objects[id].plrrad=holzchopf.checkpoint.plrrad
end

function holzchopf.checkpoint.draw(id,x,y)
	local t=getframe()
	local xs=math.sin(t/50*math.pi*2)
	local a=objects[id].alliance
	if		a==0 then setcolor(255,255,255)
	elseif	a==1 then setcolor(000,255,000)
	elseif	a==2 then setcolor(255,255,000)
	elseif	a==3 then setcolor(255,000,000)
	elseif	a==4 then setcolor(080,200,255)
	elseif	a==5 then setcolor(180,060,255)
	elseif	a==6 then setcolor(255,180,000)
	elseif	a==7 then setcolor(000,180,000)
	elseif	a==8 then setcolor(050,100,255)
	end
	setscale(xs,1)
	setrotation(0)
	setalpha(1.0)
	setblend(blend_alpha)
	drawimage(holzchopf.checkpoint.gfx_in,x,y)
	setscale(1,1)
	setrotation(t/50/4*360)
	drawimage(holzchopf.checkpoint.gfx_out,x,y)
	-- glow
	local s=0.6-math.cos(t/50/4*math.pi*2)*0.1
	particle(p_muzzle,x,y)
	particlesize(s,s)
	particlealpha(0.6)
	particlefadealpha(1)
	particlerotation(0)
	holzchopf.checkpoint.particlealliancecolor(a)
	-- pulse
	if math.mod(t,200)==0 then
		particle(p_explosionring,x,y)
		particlealpha(0.2)
		particlefadealpha(0.02)
		particlesize(0.1,0.1)
		holzchopf.checkpoint.particlealliancecolor(a)
	end
end

function holzchopf.checkpoint.update(id,x,y)
	-- check for projectile activation
	local dx,dy
	local r=objects[id].projrad
	r=r*r
	if r>=0 then
		for i=1,#projectiles do
			-- check for type ~= 0, because freed projectiles remain in array
			if getprojectiletype(i)~=0 and projectiles[i].x~=nil and projectiles[i].y~=nil then
				dx=projectiles[i].x-x
				dy=projectiles[i].y-y
				-- "hit", change alliance
				if objects[id].alliance~=getplayeralliance(0) and (dx*dx+dy*dy)<=r then
					holzchopf.checkpoint.changealliance(id,getplayeralliance(0))
				end
			end
		end
	end
	-- check for player activation
	r=objects[id].plrrad
	r=r*r
	local nearest=nil
	local neardst=nil
	if r>=0 then
		players=playertable(0,0)
		-- find nearest player
		for i=1,#players do
			dx=getplayerx(players[i])-x
			dy=getplayery(players[i])-y
			local dist=(dx*dx+dy*dy)
			if dist<=r and (neardst==nil or dist<neardst) then
				neardst=dist
				nearest=players[i]
			end
		end
		-- "touched", change alliance
		if nearest~=nil then
			if objects[id].alliance~=getplayeralliance(nearest) then
				holzchopf.checkpoint.changealliance(id,getplayeralliance(nearest))
			end
		end
	end
	-- check for underwater-deactivation
	if getobjecty(id)>getwatery()-30 and objects[id].alliance~=0 then
		holzchopf.checkpoint.changealliance(id,0)
	end
end

function holzchopf.checkpoint.damage(id,damage)
end

function holzchopf.checkpoint.particlealliancecolor(alliance)
	if		alliance==0 then particlecolor(255,255,255)
	elseif	alliance==1 then particlecolor(000,255,000)
	elseif	alliance==2 then particlecolor(255,255,000)
	elseif	alliance==3 then particlecolor(255,000,000)
	elseif	alliance==4 then particlecolor(080,200,255)
	elseif	alliance==5 then particlecolor(180,060,255)
	elseif	alliance==6 then particlecolor(255,180,000)
	elseif	alliance==7 then particlecolor(000,180,000)
	elseif	alliance==8 then particlecolor(050,100,255)
	end
end

function holzchopf.checkpoint.changealliance(id,alliance)
	x=getobjectx(id)
	y=getobjecty(id)
	-- fx 1
	for j=1,10 do
		particle(p_spark,x,y)
		holzchopf.checkpoint.particlealliancecolor(objects[id].alliance)
	end
	-- change alliance
	objects[id].alliance=alliance
	-- fx
	for j=1,10 do
		particle(p_flare,x,y)
		holzchopf.checkpoint.particlealliancecolor(objects[id].alliance)
		particlespeed(random(-1.0,1.0),random(-1.0,1.0))
	end
end